package it.disi.unitn.BAD;


import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.event.EventTarget;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Rectangle;
import javafx.scene.shape.Shape;
import javafx.stage.Stage;

import java.util.ArrayList;
import java.util.List;

public class PiastrellaApp extends Application {
    private List<Piastrella> listaPiastrelle = new ArrayList<>();
    private List<Color> paletteP = new ArrayList<>();
    private List<Color> paletteF = new ArrayList<>();
    private Info info;
    private StackPane sp;
    final private int MAX = 5;

    public static void main(String[] args) {
        launch(args);
    }

    @Override
    public void start(Stage primarystage) throws Exception {
        // Creazione degli elementi necessari per lo stage
        BorderPane root = new BorderPane();
        root.setStyle("-fx-font-family: \"Arial\""); //added by ronchet
        TextArea textBox = new TextArea();
        BorderPane container = new BorderPane();
        Button nextButton = new Button(">");
        Button prevButton = new Button("<");
        Button buyButton = new Button("Acquista");
        StackPane sp = new StackPane();
        Rectangle outer = new Rectangle(130, 130);
        Shape inner = new Rectangle(65, 65);
        // Creazione dello stage del carrello
        Carrello c = new Carrello(this);

        // Qui verranno contenuti riferimenti e informazioni per altri metodi
        info = new Info(textBox, outer, inner);
        this.sp = sp;

        // Event handler per il bottone ">"
        nextButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                // Proseguo al prossimo indice (ciclo) e aggiorno le informazioni
                info.listIndex = (info.listIndex + 1) % MAX;
                updateInfo();
            }
        });

        // Event handler per il bottone "<"
        prevButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                // Proseguo al prossimo indice (ciclo) e aggiorno le informazioni
                if (info.listIndex - 1 < 0) {
                    info.listIndex = MAX - 1;
                }
                else {
                    info.listIndex = (info.listIndex - 1) % MAX;
                }
                updateInfo();
            }
        });

        // Event handler per il bottone "Acquista"
        buyButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent actionEvent) {
                // Aggiungo un nuovo acquisto allo stage del carrello
                Piastrella current = listaPiastrelle.get(info.listIndex);
                c.addEntry(current);
            }
        });

        // Binding degli eventi ai due pezzi di piastrella (per il cambio di colore)
        outer.setOnMouseClicked(new ChangeColorHandler());
        inner.setOnMouseClicked(new ChangeColorHandler());
        outer.setId("outer");
        inner.setId("inner");

        root.setTop(textBox);
        root.setCenter(container);
        root.setBottom(buyButton);
        // .setAlignment(Pos.CENTER) non sembrava funzionare
        root.setMargin(buyButton, new Insets(0, 0, 0, 130));
        root.setStyle("-fx-font-family: \"Arial\""); //added by ronchet

        container.setLeft(prevButton);
        container.setRight(nextButton);
        container.setCenter(sp);

        textBox.setMaxSize(320, 65);
        textBox.setEditable(false);

        sp.getChildren().addAll(outer, inner);

        // Inizializzazione di tutte le piastrelle, e conseguente aggiornamento alla prima piastrella
        initPiastrelle();
        updateInfo();

        // Visualizzazione di entrambi gli stage
        primarystage.setTitle("BAD PROJECT");
        primarystage.setScene(new Scene(root, 320, 240));
        primarystage.show();

        c.setX(260);
        c.show();
    }

    private void initPiastrelle() {
        // Inserisco le piastrelle
        listaPiastrelle.add(new Piastrella("N1", 5, Materiale.CERAMICA, false));
        listaPiastrelle.add(new Piastrella("C2", 6, Materiale.TERRACOTTA, true));
        listaPiastrelle.add(new PiastrellaBi("B3", 10, Materiale.LAMINATO,true, Figura.QUADRATO));
        listaPiastrelle.add(new Piastrella("C4", 8, Materiale.CERAMICA, true));
        listaPiastrelle.add(new PiastrellaBi("B5", 16, Materiale.CERAMICA, true, Figura.CERCHIO));

        // Inizializzo le palette di colori
        paletteP.add(Color.LIGHTCYAN);
        paletteP.add(Color.LIGHTPINK);
        paletteP.add(Color.LEMONCHIFFON);

        paletteF.add(Color.RED);
        paletteF.add(Color.WHITE);
        paletteF.add(Color.INDIGO);
        paletteF.add(Color.GOLD);

        // Assegna colori all'avvio
        for (Piastrella p : listaPiastrelle) {
            p.colore = paletteP.get(0);

            if (p instanceof PiastrellaBi) {
                ((PiastrellaBi) p).coloreInterno = paletteF.get(0);
            }
        }
    }

    private void updateInfo() {
        // Reset della piastrella
        sp.getChildren().clear();

        Piastrella p = listaPiastrelle.get(info.listIndex);

        info.text.setText("Codice: " + p.codice + "\nMateriale: " + p.materiale +
                "\nCosto: " + p.costo);

        // Aggiorno la piastrella esterna
        info.outer = new Rectangle(130, 130);
        info.outer.setStroke(Color.BLACK);
        info.outer.setFill(p.colore);

        // SBAGLIATISSIMO! invece che mettere la logica dentro le classi, la tiene esterna e sulla base del tipo
        // fa una cosa oppure l'altra. Ma il polimorfismo cosa l'hanno inventato a fare?

        if (p instanceof PiastrellaBi) {
            // Assegno la nuova forma alla piastrella interna
            if (((PiastrellaBi) p).figuraInterna.equals(Figura.QUADRATO)) {
                info.inner = new Rectangle(65, 65);
            }
            else {
                info.inner = new Circle(0, 0, 33);
            }

            // Aggiorno la piastrella interna e rifaccio il binding agli eventi
            info.inner.setFill(((PiastrellaBi) p).coloreInterno);
            info.inner.setStroke(Color.BLACK);

            info.outer.setOnMouseClicked(new ChangeColorHandler());
            info.inner.setOnMouseClicked(new ChangeColorHandler());
            info.outer.setId("outer");
            info.inner.setId("inner");

            sp.getChildren().addAll(info.outer, info.inner);
        }
        else {
            // Nel caso la piastrella è monocolore
            info.outer.setOnMouseClicked(new ChangeColorHandler());
            info.outer.setId("outer");
            sp.getChildren().add(info.outer);
        }
    }

    // Classe per gestire il cambio di colore delle piastrelle
    class ChangeColorHandler implements EventHandler<MouseEvent> {
        @Override
        public void handle(MouseEvent mouseEvent) {
            EventTarget target = mouseEvent.getTarget();
            Piastrella current = listaPiastrelle.get(info.listIndex);

            // QUI DI NUOVO VA, SULLA BASE DEL VALORE DI UNA PROPRIETA', A COMPORTARSI IN UN MODO OPPURE IN UN ALTRO...
            // Controllo se la piastrella può cambiare colore
            if (current.sceltaColore) {
                // Controllo che tipo di piastrella sto gestendo (interna/esterna)
                if (((Node)target).getId().equals("outer")) {
                    // Aggiornamento dell'indice del colore e conseguente aggiornamento del colore
                    info.palettePIndex = (info.palettePIndex + 1) % paletteP.size();
                    ((Shape)target).setFill(paletteP.get(info.palettePIndex));
                }
                else {
                    // Aggiornamento dell'indice del colore e conseguente aggiornamento del colore
                    info.paletteFIndex = (info.paletteFIndex + 1) % paletteF.size();
                    ((Shape)target).setFill(paletteF.get(info.paletteFIndex));
                    ((PiastrellaBi) current).coloreInterno = paletteF.get(info.paletteFIndex);
                }

                // Aggiorno il colore in memoria
                current.colore = paletteP.get(info.palettePIndex);
            }
        }
    }
}
